#include "clipper.h"

#include <GL/gl.h>
#include <GL/glu.h>

#include <glib.h>

static void
clip_node_render (Node *node)
{
    ClipNode *clip_node = (ClipNode *)node;

    /* coordinate system */
    
#if 0
    glEnable (GL_BLEND);
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    glLineWidth (10.0);
    
    glBegin (GL_LINES);
    
    glColor4f (1.0, 0.0, 1.0, 1.0);
    
    glVertex2f (0.0, 0.0);
    glVertex2f (1.0, 0.0);
    
    glColor4f (0.0, 1.0, 0.0, 1.0);
    
    glVertex2f (0.0, 0.0);
    glVertex2f (0.0, 1.0);

    glEnd();
#endif
    
    /* We can do this more efficiently by adding a
     *
     *   node_get_texture (Node, x, y, w, h, *texture, *tx, *ty, *tw, *th)
     *
     * for items that return a texture we can then do clipping by just
     * setting texture coordinates.
     *
     * We could even have the possibility of returning multiple textures.
     */
    
    double eqn0[4] = {  1.0, 0.0, 0.0,  -clip_node->x };
    double eqn1[4] = {  -1.0, 0.0, 0.0,  (clip_node->x + clip_node->w) };
    double eqn2[4] = { 0.0,  1.0, 0.0, -clip_node->y };
    double eqn3[4] = { 0.0,  -1.0, 0.0, (clip_node->y + clip_node->h) };

    glClipPlane(GL_CLIP_PLANE0, eqn0);
    glClipPlane(GL_CLIP_PLANE1, eqn1);
    glClipPlane(GL_CLIP_PLANE2, eqn2);
    glClipPlane(GL_CLIP_PLANE3, eqn3); 

    glPushMatrix ();
    
    glEnable (GL_CLIP_PLANE0);
    glEnable (GL_CLIP_PLANE1);
    glEnable (GL_CLIP_PLANE2);
    glEnable (GL_CLIP_PLANE3);
    
    clip_node->child->render (clip_node->child);

    glPopMatrix();
    
    glDisable (GL_CLIP_PLANE0);
    glDisable (GL_CLIP_PLANE1);
    glDisable (GL_CLIP_PLANE2);
    glDisable (GL_CLIP_PLANE3);
}

ClipNode *
clip_node_new (double x, double y,
	       double width, double height,
	       Node *child)
{
    ClipNode *clip_node = g_new0 (ClipNode, 1);

    clip_node->x = x;
    clip_node->y = y;
    clip_node->w = width;
    clip_node->h = height;

    clip_node->child = child;

    clip_node->node.render = clip_node_render;
    
    return clip_node;
}
