#include <glib.h>
#include <GL/gl.h>
#include "node.h"
#include "rotation.h"

static void
rotation_compute_extents (Node *node,
			  Extents *extents)
{
    extents->x = 0.0;
    extents->y = 0.0;
    extents->z = 0.0;
    extents->width = 1.0;
    extents->height = 1.0;
    extents->depth = 1.0;
}

static void
rotation_render (Node *node)
{
    Rotation *rotation = (Rotation *)node;
    GList *list;
    GLboolean depth_mask;
    gdouble depth;
    
    glPushMatrix();

    glRotatef (rotation->angle,
	       rotation->x,
	       rotation->y,
	       rotation->z);

    rotation->child->render (rotation->child);

    glPopMatrix();
}

Rotation *
rotation_new (Node *child)
{
    Rotation *rotation = g_new0 (Rotation, 1);

    rotation->node.render = rotation_render;
    rotation->node.compute_extents = rotation_compute_extents;

    rotation->angle = 0.0;
    rotation->x = 0.0;
    rotation->y = 0.0;
    rotation->z = 1.0;

    rotation->child = child;
    
    return rotation;
}

void
rotation_set_rotation (Rotation *rotation,
			  gdouble	     angle,
			  gdouble	     x,
			  gdouble	     y,
			  gdouble      z)
{
    rotation->angle = angle;
    rotation->x = x;
    rotation->y = y;
    rotation->z = z;
}

