#include <glib.h>
#include <GL/gl.h>
#include "node.h"
#include "stacker.h"

static void
stacker_compute_extents (Node *node,
			 Extents *extents)
{
    extents->x = 0.0;
    extents->y = 0.0;
    extents->z = 0.0;
    extents->width = 1.0;
    extents->height = 1.0;
    extents->depth = 1.0;
}

static void
stacker_render (Node *node)
{
    Stacker *stacker = (Stacker *)node;
    GList *list;
    GLboolean depth_mask;
    gdouble depth;
    
    glPushMatrix();

    /* First draw everything without updating the depth buffer.
     * This ensures that children in the same plane will
     * overdraw each other, but still get clipped by existing
     * depth buffered drawing.
     *
     * This won't work if the stacker contains 3D children, so we
     * assume it doesn't.
     */

    glEnable (GL_BLEND);
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    
    for (list = g_list_last (stacker->children); list != NULL; list = list->prev)
    {
	Node *child = list->data;

	/* sort of crack, but we probably can't rely on
	 * depth buffering working.
	 */
	
	glTranslatef (0.0, 0.0, 0.0001);
	
	child->render (child);
    }
    
    glPopMatrix();
}

Stacker *
stacker_new (void)
{
    Stacker *stacker = g_new0 (Stacker, 1);

    stacker->node.render = stacker_render;
    stacker->node.compute_extents = stacker_compute_extents;
    
    return stacker;
}

void
stacker_add_child (Stacker *stacker,
		   Node *child)
{
    stacker->children = g_list_prepend (stacker->children, child);
}

void
stacker_remove_child (Stacker *stacker,
		      Node *child)
{
    stacker->children = g_list_remove (stacker->children, child);
}
