#include <X11/X.h>
#include <X11/Xutil.h>
#include <X11/extensions/Xcomposite.h>
#include <X11/extensions/Xdamage.h>
#include <X11/extensions/XShm.h>

#include "ws.h"

#ifndef _WSINT_H_
#define _WSINT_H_

typedef struct
{
    gboolean	available;
    int		event_base;
    int		error_base;
} Extension;

/*
 * Window system object
 */
typedef struct Trap Trap;

struct Trap
{
    int begin_serial;
    int end_serial;
    
    Trap *next;
};

typedef int (* ErrorHandlerFunc) (Display *, XErrorEvent *);

struct Ws
{
    Display *	xdisplay;
    
    int		n_screens;
    WsScreen **	screens;
    
    GHashTable *xresources;
    
    Extension	damage;
    Extension	composite;
    Extension	shm;
    Extension   test;
    Extension	fixes;

    Trap	*traps;

    ErrorHandlerFunc	old_handler;
};

/*
 * Region object
 */

struct WsRegion
{
    Ws *		ws;
    XserverRegion	xregion;

    guint		ref_count;
};


/*
 * Visual object
 */

struct WsVisual
{
    Visual	       *visual;
};

/*
 * Drawable object
 */
struct WsDrawable
{
    Ws *		ws;
    XID			xid;
    
    Damage		xdamage;
    WsWindowDamageCallback
			damage_cb;
    gpointer		damage_data;

    int			ref_count;
};

void _ws_drawable_process_event (WsDrawable *window,
				 XEvent     *event);
void _ws_drawable_init (WsDrawable *drawable,
			Ws *	    ws,
			XID	    xid);

/*
 * Pixmap object
 */
struct WsPixmap
{
    WsDrawable		drawable;
};

WsPixmap *_ws_pixmap_ensure (Ws     *ws,
			     Pixmap  xwindow);

/*
 * Shared memory pixmap object
 */
struct WsShmPixmap
{
    WsPixmap		pixmap;
    
    XShmSegmentInfo	shm_info;

    WsFormat		format;
    guint		width;
    guint		height;
    XImage *		image;
    guint		bytes_per_row;
};

/*
 * Window object
 */
struct WsWindow
{
    WsDrawable		drawable;

    WsConfigureCallback configure_callback;
    gpointer		configure_data;
};
    
struct WsWindowClass
{
#if 0
    GObjectClass	parent;    
#endif
};

void      _ws_window_process_event (WsWindow   *window,
				    XEvent     *xevent);
WsWindow *_ws_window_ensure        (Ws         *ws,
				    Window      xwindow);

/*
 * Screen object
 */
struct WsScreen
{
    Ws     *ws;
    Screen *xscreen;
};

#endif
