#include "wsint.h"

WsRegion *
ws_region_new (Ws *ws)
{
    WsRegion *region = g_new0 (WsRegion, 1);

    region->ws = ws;
    region->xregion = XFixesCreateRegion (ws->xdisplay, NULL, 0);
    region->ref_count = 1;
    
    return region;
}

void
ws_region_unref (WsRegion *region)
{
    if (--region->ref_count == 0)
    {
	XFixesDestroyRegion (region->ws->xdisplay,
			     region->xregion);
	g_free (region);
    }
}

void
ws_region_union (WsRegion *region,
		 WsRegion *other)
{
    XFixesUnionRegion (region->ws->xdisplay,
		       region->xregion,
		       region->xregion,
		       other->xregion);
}

WsRectangle *
ws_region_query_rectangles (WsRegion *region,
			    guint    *n_rectangles)
{
    int n_rects = 0;
    XRectangle *xrects;
    
    xrects = XFixesFetchRegion (region->ws->xdisplay,
			       region->xregion, &n_rects);

    if (n_rectangles)
	*n_rectangles = n_rects;
	
    if (xrects)
    {
	WsRectangle *ws_rects = g_new (WsRectangle, n_rects);
	int i;

	for (i = 0; i < n_rects; ++i)
	{
	    ws_rects[i].x = xrects[i].x;
	    ws_rects[i].y = xrects[i].y;
	    ws_rects[i].width = xrects[i].width;
	    ws_rects[i].height = xrects[i].height;
	}

	return ws_rects;
    }

    return NULL;
}
